#BEGIN COPYRIGHT
#*************************************************************************
#
# Licensed Materials - Property of IBM
# 5725-C94, 5725-C95, 5725-C96
# (C) Copyright IBM Corporation 2014, 2015. All Rights Reserved.
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#*************************************************************************
import os,sys,re,shutil
from com.ibm.ISecurityUtilityImpl import PasswordUtil
from com.ibm.websphere.management.configservice import ConfigServiceFactory
from com.ibm.websphere.management.configservice import ConfigServiceHelper
from com.ibm.websphere.management import Session

debug="false"
#--------------------------------------------------------------
#Print trace according to debug setting
#--------------------------------------------------------------

def trace(message):
    if(debug == "true"):
        print(message)
#--------------------------------------------------------------
# Helper Class to normalize the string for comparision
#--------------------------------------------------------------
def wsadminToList(inStr):
    outList=[]
    if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
        tmpList = inStr[1:-1].split() #splits space-separated lists,
    else:
        tmpList = inStr.split("\n")   #splits for Windows or Linux
    for item in tmpList:
        item = item.rstrip();         #removes any Windows "\r"
        if (len(item)>0):
            outList.append(item)
    return outList;
#endDef

#------------------------------------------------------------------------------
# get the scope Id according to cellname ,nodename, servername, clustername
#------------------------------------------------------------------------------
def getScopeId(cellName,nodeName,serverName,clusterName):    
    scopeId=None;
    if (cellName == None):
        scopeId=None
    elif (nodeName ==None and clusterName == None):
        scopeId=AdminConfig.getid('/Cell:' + cellName);
    elif (serverName == None and clusterName == None):
        scopeId=AdminConfig.getid('/Cell:' + cellName + '/Node:' + nodeName);
    elif (serverName == None):
        scopeId=AdminConfig.getid('/Cell:' + cellName + '/ServerCluster:' + clusterName);
    else:
        scopeId=AdminConfig.getid('/Cell:' + cellName + '/Node:' +nodeName + '/Server:' + serverName);   
    return scopeId;
#endDef

def extractFileName(objectID):
    fileName = None
    regexp = re.compile('.*xml#(.*_[0-9]+).*')
    match = re.match(regexp,objectID)
    if(match != None):
        fileName = str(match.group(1))
    else:
        print "File name not found in " + objectID
    return str(fileName)
    
#------------------------------------------------------------------------------
# Handle parsed parameter for specifying scope
#------------------------------------------------------------------------------
def pasrseParam(params):
    result = {}
    index = 0
    while(index < len(params)):
        result[params[index]] = params[index+1]
        index = index + 2
    return result

#------------------------------------------------------------------------------
# Print usage
#------------------------------------------------------------------------------
def usage():
    print "Usage:\n"
    print "\tList all data source:"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -listDataSources\n"
    print "\tExport data source by JNDI name in cell scope:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportDataSource <Datasource_JNDI> <output_dir> -cell <cell_name>\n"
    print "\tExport data source by JNDI name in node scope:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportDataSource <Datasource_JNDI> <output_dir> -cell <cell_name> -node <node_name>\n"
    print "\tExport data source by JNDI name in server scope:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportDataSource <Datasource_JNDI> <output_dir> -cell <cell_name> -node <node_name> -server <server_name>\n"
    print "\tExport data source by JNDI name in cluster scope:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportDataSource <Datasource_JNDI> <output_dir> -cell <cell_name> -cluster <clusterName>\n"
    print "\tExport all data sources in cell scope:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportAllDataSources <output_dir> -cell <cell_name>\n"
    print "\tExport all data sources in node scope:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportAllDataSources <output_dir> -cell <cell_name> -node <node_name>\n"
    print "\tExport all data sources in server scope:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportAllDataSources <output_dir> -cell <cell_name> -node <node_name> -server <server_name>\n"
    print "\tExport all data sources in cluster scope:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportAllDataSources <output_dir> -cell <cell_name> -cluster <clusterName>\n"
    print "\tList all auth alias:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -listAuthAliases\n"    
    print "\tExport auth alias by name:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportAuthAlias <Auth_Alias_Name> <output_dir>\n"
    print "\tExport all auth aliases:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportAllAuthAliases <output_dir>\n"
    print "\tExport all SSL setting:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportSSLSettings <output_dir>\n"
    print "\tExport java security file java.security:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -lang jython -f exportWASConfig.py -exportJavaSecurityFile <output_dir>\n"
    sys.exit()

#------------------------------------------------------------------------------
# Export all database under specified scope
#------------------------------------------------------------------------------
def exportAllDataSources(outputFilePath,scopeId):    
    if(not os.path.exists(outputFilePath)):
        os.makedirs(outputFilePath)
    dataSourceList=AdminConfig.list("DataSource",scopeId)
    dataSources = wsadminToList(dataSourceList)
    sumPropertyFile = None          
    if(len(dataSources) > 0):
        sumPropertyFile = open(os.path.join(outputFilePath,"exportDS.properties"),"w")        
        for ds in dataSources:       
            fileName = extractFileName(ds) 
            trace("Output file name:" + fileName)
            jndiName=AdminConfig.showAttribute(ds,"jndiName")      
            if(jndiName):            
                outputPropFileName = os.path.join(outputFilePath,fileName + ".properties")           
                outputObjFileName = os.path.join(outputFilePath,fileName + ".obj")
                trace("About to export properties of data source with JNDI " + str(jndiName))            
    #             exportDataSource(ds,outputPropFileName)
                exportDataSourceObj(ds,outputObjFileName)
                sumPropertyFile.write("#Exported Data Source Object with JNDI name -- " + jndiName + "\n")
                sumPropertyFile.write(jndiName + ".ObjectFileName=" + fileName+".obj\n")
                sumPropertyFile.write(jndiName + ".ObjectType=DataSource\n")
                jdbcProvider = AdminConfig.showAttribute(ds,"provider")
                if(jdbcProvider != None):
                    sumPropertyFile.write(jndiName + ".Provider=" + str(AdminConfig.showAttribute(jdbcProvider,"name")) + "\n")   
                    sumPropertyFile.write(jndiName + ".ProviderType=" + str(AdminConfig.showAttribute(jdbcProvider,"providerType")) + "\n")
                resourceAdapter = AdminConfig.showAttribute(ds,"relationalResourceAdapter")
                if(resourceAdapter != None):
                    sumPropertyFile.write(jndiName + ".ResourceAdapter=" + str(AdminConfig.showAttribute(resourceAdapter,"name")) + "\n\n")           
                print "INFO:Data source with JNDI "  + str(jndiName) + " was exported to file " + fileName + ".properties" + " and " + fileName + ".obj under " + outputFilePath
        print "INFO:Check file " + os.path.join(outputFilePath,"exportDS.properties") + " for export summary."
    else:
        print "INFO:No data sources were found."    
    if(sumPropertyFile):
        sumPropertyFile.close()

#------------------------------------------------------------------------------
# List all data source name/JNDI in current cell
#------------------------------------------------------------------------------
def listDataSources():
    dataSourceList=AdminConfig.list("DataSource")
    dataSources = wsadminToList(dataSourceList)
    index = 1
    print "Data source list:\n"    
    for ds in dataSources:
        trace("Data source:"+str(ds))
        findex = ds.find("(")
        lindex = ds.find("resources.xml#")
        scopeId = ds[findex:lindex]
        dsName=AdminConfig.showAttribute(ds,"name")
        jndiName=AdminConfig.showAttribute(ds,"jndiName")
        providerType=AdminConfig.showAttribute(ds,"providerType")
        print index,"\nScope:",scopeId,"\nName:",dsName,"\nJNDI:",jndiName,"\nProviderType:",providerType,"\n"
        index = index + 1
        
#------------------------------------------------------------------------------
# List all auth alias
#------------------------------------------------------------------------------
def listAuthAliases():
    globalSecurity = AdminConfig.list("Security" )
    authAliases = AdminConfig.list("JAASAuthData",globalSecurity)   
    print "Auth Alias list:"
    if len(authAliases) > 0 :
        for authAlias in wsadminToList(authAliases):
            authAliasName = AdminConfig.showAttribute(authAlias,"alias")
            print "Name:",authAliasName           
            
#------------------------------------------------------------------------------
# Export data source by JNDI name in specified scope
#------------------------------------------------------------------------------            
def exportDataSourceByJNDI(dsJNDI,outputFilePath,scopeId):    
    if(not os.path.exists(outputFilePath)):
        os.makedirs(outputFilePath)
    dataSourceList=AdminConfig.list("DataSource",scopeId)
    dataSources = wsadminToList(dataSourceList)
    sumPropertyFile = None   
    found = "false" 
    for datasource in dataSources:      
        fileName = extractFileName(datasource)  
        #index = datasource.rfind(".xml#")
        #fileName = datasource[index+5:-2]
        jndiName=AdminConfig.showAttribute(datasource,"jndiName")
#         print "JNDI name:" + jndiNameAdmin
        if (jndiName != None and jndiName == dsJNDI):     
            found = "true"
            sumPropertyFile = open(os.path.join(outputFilePath,"exportDS.properties"),"w")         
            outputPropFile = os.path.join(outputFilePath,fileName + ".properties")
            outputObjFileName = os.path.join(outputFilePath,fileName + ".obj")           
#             exportDataSource(datasource,outputPropFile)
            exportDataSourceObj(datasource,outputObjFileName)
            sumPropertyFile.write("#Exported Data Source Object with JNDI name -- " + jndiName + "\n")
            sumPropertyFile.write(jndiName + ".ObjectFileName=" + fileName+".obj\n")
            sumPropertyFile.write(jndiName + ".ObjectType=DataSource\n")
            jdbcProvider = AdminConfig.showAttribute(datasource,"provider")
            if(jdbcProvider != None):
                sumPropertyFile.write(jndiName + ".Provider=" + str(AdminConfig.showAttribute(jdbcProvider,"name")) + "\n")   
                sumPropertyFile.write(jndiName + ".ProviderType=" + str(AdminConfig.showAttribute(jdbcProvider,"providerType")) + "\n")
            resourceAdapter = AdminConfig.showAttribute(datasource,"relationalResourceAdapter")
            if(resourceAdapter != None):
                sumPropertyFile.write(jndiName + ".ResourceAdapter=" + str(AdminConfig.showAttribute(resourceAdapter,"name")) + "\n\n")
            print "INFO:Data source with JNDI "  + str(jndiName) + " was exported to file " + fileName + ".obj under " + outputFilePath
            print "INFO:Check file " + os.path.join(outputFilePath,"exportDS.properties") + " for export summary."
    if (found == "false"):
        print "INFO:Data source with JNDI name " + dsJNDI + " was not found.Please check the inputed auth alias name."
    if(sumPropertyFile):
        sumPropertyFile.close()        

#------------------------------------------------------------------------------
# Export data source 
#------------------------------------------------------------------------------            
def exportDataSourceObj(ds,outputObjFileName):
    outputObjFile = open(outputObjFileName,"w")
    dbObj = AdminConfig.showall(ds)
    outputObjFile.write(str(dbObj))
    if(outputObjFile):
        outputObjFile.close()

#------------------------------------------------------------------------------
# Export data source
#------------------------------------------------------------------------------        
def exportDataSource(datasource,outputFilePath):
    simpleAttrTypes = ["long","int","String","boolean"]
    outputFile = open(outputFilePath,"w")
    outputFile.write("#Exported properties of data source with JNDI name - "+str(AdminConfig.showAttribute(datasource,"jndiName"))+"\n")    
    outputFile.write("\n")
    outputFile.write("#General properties\n")
    dsAttributes = wsadminToList(AdminConfig.attributes("DataSource"))
    for dsAttr in dsAttributes:
        trace("Data source general property --" + str(dsAttr))
        if(dsAttr):
            offsetOfFirstSpace = dsAttr.find(" ")
            attributeName = dsAttr[0:offsetOfFirstSpace]
            attributeType = dsAttr[offsetOfFirstSpace+1:]
            trace("Name:" + attributeName + "  Type:" + attributeType)
            if(attributeType in simpleAttrTypes) or attributeType.startswith("ENUM"):
                attributeValue=AdminConfig.showAttribute(datasource,attributeName);                            
                trace("Value:"+str(attributeValue))
                outputFile.write(str(attributeName)+ "=" + str(attributeValue)+"\n")
    outputFile.write("\n")
    outputFile.write("#Custom properties\n")
    propSet = AdminConfig.showAttribute(datasource,"propertySet")
    resoucepropertiesList = wsadminToList(AdminConfig.list("J2EEResourceProperty", propSet))
    for propid in resoucepropertiesList:
        curPropName=AdminConfig.showAttribute(propid,"name")
        curPropValue=AdminConfig.showAttribute(propid,"value")
        curPropType=AdminConfig.showAttribute(propid,"type")
        curPropDesc=AdminConfig.showAttribute(propid,"description")
        curPropRequired=AdminConfig.showAttribute(propid,"required")
        curPropIgnore=AdminConfig.showAttribute(propid,"ignore")
        curPropConfidential=AdminConfig.showAttribute(propid,"confidential")
        curPropSupportDynamic=AdminConfig.showAttribute(propid,"supportsDynamicUpdates")
        outputFile.write(str(curPropName) + "=" + str(curPropValue))
        outputFile.write("  #(type="+str(curPropType)+",required="+str(curPropRequired)+",ignore="+str(curPropIgnore) +",confidential=" + str(curPropConfidential)+",supportsDynamicUpdates=" + str(curPropSupportDynamic)+")\n")
    outputFile.write("\n")
    outputFile.write("#Connection pool properties\n")
    connectionPool = AdminConfig.showAttribute(datasource,"connectionPool")
    connectionPoolAttributes = wsadminToList(AdminConfig.attributes("ConnectionPool"))
    for connectionPoolAttribute in connectionPoolAttributes:
        if(connectionPoolAttribute):
            offsetOfFirstSpace = connectionPoolAttribute.find(" ")
            cpAttributeName = connectionPoolAttribute[0:offsetOfFirstSpace]
            cpAttributeType = connectionPoolAttribute[offsetOfFirstSpace+1:]
            trace("Name:" + cpAttributeName + "  Type:" + cpAttributeType)
            if(cpAttributeType in simpleAttrTypes) or cpAttributeType.startswith("ENUM"):
                cpAttributeValue=AdminConfig.showAttribute(connectionPool,cpAttributeName);                            
                trace("Value:"+str(cpAttributeValue))
                outputFile.write(str(cpAttributeName)+ "=" + str(cpAttributeValue)+"\n")
    outputFile.write("\n")
    if(outputFile):
        outputFile.close()
       
def exportAuthAliasObj(authAlias,outputObjFileName,namePasswordMap):
    encodedPassword = None
    outputObjFile = open(outputObjFileName,"w")
    authAliasObj = AdminConfig.showall(authAlias)
    if(namePasswordMap.has_key(str(AdminConfig.showAttribute(authAlias,"alias")))):
        password = namePasswordMap[str(AdminConfig.showAttribute(authAlias,"alias"))]
        encodedPassword = PasswordUtil.passwordEncode(password)
    if(authAliasObj.rfind("*****") > 0 and encodedPassword != None):
        authAliasObj = authAliasObj.replace("*****",encodedPassword)       
    outputObjFile.write(str(authAliasObj))
    if(outputObjFile):
        outputObjFile.close()

#------------------------------------------------------------------------------
# Export auth alias 
#------------------------------------------------------------------------------
def exportAuthAlias(authAlias,outputPropFile):   
    outputFile = open(outputPropFile,"w")
    outputFile.write("#Exported properties of auth alias - "+str(AdminConfig.showAttribute(authAlias,"alias"))+"\n")       
    outputFile.write("alias=" + str(AdminConfig.showAttribute(authAlias,"alias")) + "\n")
    outputFile.write("userId=" + str(AdminConfig.showAttribute(authAlias,"userId")) + "\n")
    outputFile.write("password=" + str(AdminConfig.showAttribute(authAlias,"password")) + "\n")
    outputFile.write("description=" + str(AdminConfig.showAttribute(authAlias,"description")) + "\n")          
    if(outputFile):
        outputFile.close()

def getAuthAliasNamePasswordMapping():
    namePwMap = {}
    session=Session()
    configService=ConfigServiceFactory.getConfigService()
    queryObject=ConfigServiceHelper.createObjectName(None,"JAASAuthData")
    authAliases = configService.queryConfigObjects(session,None,queryObject,None)
    for authAlias in authAliases:
        name = configService.getAttribute(session,authAlias,"alias")
        password = configService.getAttribute(session,authAlias,"password")
        namePwMap[name] = password
    return namePwMap
            
#------------------------------------------------------------------------------
# Export all auth alias
#------------------------------------------------------------------------------
def exportAllAuthAliases(outputDir):
    if(not os.path.exists(outputDir)):
        os.makedirs(outputDir)
    namePasswordMap = getAuthAliasNamePasswordMapping()
    globalSecurity = AdminConfig.list("Security" )
    authAliases = AdminConfig.list("JAASAuthData",globalSecurity)   
    if len(authAliases) > 0 :
        sumPropertyFile = open(os.path.join(outputDir,"exportAlias.properties"),"w")
        for authAlias in wsadminToList(authAliases):
            fileName = extractFileName(authAlias) 
            trace("Output file name:" + fileName)
            outputPropFile = os.path.join(outputDir,fileName + ".properties")
            outputObjFileName = os.path.join(outputDir,fileName + ".obj")           
#             exportAuthAlias(authAlias,outputPropFile)
            exportAuthAliasObj(authAlias,outputObjFileName,namePasswordMap)
            sumPropertyFile.write("#Exported Auth Alias\n")
            sumPropertyFile.write(str(AdminConfig.showAttribute(authAlias,"alias")) + ".ObjectFileName=" + fileName+".obj\n")
            sumPropertyFile.write(str(AdminConfig.showAttribute(authAlias,"alias")) + ".ObjectType=JAASAuthData\n\n") 
            print "INFO:Auth Alias "  + str(AdminConfig.showAttribute(authAlias,"alias")) + " was exported to file " + fileName +".obj under " + outputDir
        print "INFO:Check file " + os.path.join(outputDir,"exportAlias.properties") + " for export summary."      
        if(sumPropertyFile):
            sumPropertyFile.close()   
    else:
        print "INFO:No auth aliases were found."         
            
#------------------------------------------------------------------------------
# Export auth alias by name
#------------------------------------------------------------------------------
def exportAuthAliasByName(authAliasName,outputDir):
    if(not os.path.exists(outputDir)):
        os.makedirs(outputDir)
    namePasswordMap = getAuthAliasNamePasswordMapping()
    globalSecurity = AdminConfig.list("Security" )
    authAliases = AdminConfig.list("JAASAuthData",globalSecurity)   
    found = "false"
    if len(authAliases) > 0 :
        for authAlias in wsadminToList(authAliases):
            fileName = extractFileName(authAlias) 
            trace("Output file name:" + fileName)
            aliasName = AdminConfig.showAttribute(authAlias,"alias")
            if(aliasName == authAliasName):
                found = "true"
                sumPropertyFile = open(os.path.join(outputDir,"exportAlias.properties"),"w")
                outputPropFile = os.path.join(outputDir,fileName + ".properties")
                outputObjFileName = os.path.join(outputDir,fileName + ".obj")           
#                 exportAuthAlias(authAlias,outputPropFile)
                exportAuthAliasObj(authAlias,outputObjFileName,namePasswordMap)
                sumPropertyFile.write("#Exported Auth Alias\n")
                sumPropertyFile.write(str(AdminConfig.showAttribute(authAlias,"alias")) + ".ObjectFileName=" + fileName+".obj\n")
                sumPropertyFile.write(str(AdminConfig.showAttribute(authAlias,"alias")) + ".ObjectType=JAASAuthData\n\n")       
                print "INFO:Auth Alias "  + str(AdminConfig.showAttribute(authAlias,"alias")) + " was exported to file " + fileName + ".obj under " + outputDir
                print "INFO:Check file " + os.path.join(outputDir,"exportAlias.properties") + " for export summary."       
                if(sumPropertyFile):
                    sumPropertyFile.close()         
    if (found == "false"):
        print "INFO:Auth alias with name " + authAliasName + " was not found.Please check the inputed JNDI name."
#------------------------------------------------------------------------------
# Export ssl setting in the cell
#------------------------------------------------------------------------------
def exportSSLSettings(outputDir):
    if(not os.path.exists(outputDir)):
        os.makedirs(outputDir)
    sslConfigs = AdminTask.listSSLConfigs('[-all true -displayObjectName true ]')
    sumPropertyFile = open(os.path.join(outputDir,"exportSSL.properties"),"w")
    sslConfigs = wsadminToList(sslConfigs)
    if(len(sslConfigs) > 0):
        for sslConfig in sslConfigs:
            alias = AdminConfig.showAttribute(sslConfig,"alias")        
            managementScope = AdminConfig.showAttribute(sslConfig,"managementScope")
            scopeName = AdminConfig.showAttribute(managementScope,"scopeName")
            setting = AdminConfig.showAttribute(sslConfig,"setting")
            securityLevel = AdminConfig.showAttribute(setting,"securityLevel")
    #         sslCiphers = AdminTask.listSSLCiphers('[-sslConfigAliasName ' + alias + ' -scopeName ' + scopeName + ' -securityLevel ' + securityLevel + ' ]')
    #         outputPropFile = open(os.path.join(outputDir,alias + ".properties"),"w")
            outputObjFile = open(os.path.join(outputDir,alias + ".obj"),"w")
            
    #         outputPropFile.write("securityLelve=" + securityLevel +"\n")
    #         outputPropFile.write("managementScope=" + managementScope+"\n")
    #         outputPropFile.write("scopeName=" + scopeName+"\n")
    #         outputPropFile.write("sslCiphers=" + str(wsadminToList(sslCiphers))+"\n")        
            outputObjFile.write(AdminConfig.showall(setting))        
            sumPropertyFile.write("#Exported SLL Setting\n")
            sumPropertyFile.write(alias + ".ObjectFileName=" + alias + ".obj\n")
            sumPropertyFile.write(alias + ".ScopeName=" + scopeName + "\n")
            print "INFO:SSL setting "  + alias + " was exported to file " + alias + ".obj under " + outputDir
    #         if(outputPropFile):
    #             outputPropFile.close()
            if(outputObjFile):
                outputObjFile.close()
        print "INFO:Check file " + os.path.join(outputDir,"exportSSL.properties") + " for export summary."       
        if(sumPropertyFile):
            sumPropertyFile.close()
    else:
        print "INFO:No SSL setting was found."

def exportJavaSecurityFile(outputDir):    
    if(not os.path.exists(outputDir)):
        os.makedirs(outputDir)    
    defaultJDKPath = getDefaultJDKPath()
    jreSecurityFilePath = os.path.join(defaultJDKPath,'jre/lib/security/java.security')
    exportedFilePath = os.path.join(outputDir,'java.security')
    shutil.copyfile(jreSecurityFilePath, exportedFilePath)
    print "INFO:" + jreSecurityFilePath + " was exported to " + exportedFilePath
                  
def getDefaultJDKPath():      
    from java.util import Properties
    from java.io import FileInputStream
    wasInstallRoot = java.lang.System.getProperty('was.install.root')
    cmdDefaultSDKPropertyFilePath = os.path.join(wasInstallRoot,'properties/sdk/cmdDefaultSDK.properties')
    cmdDefaultSDKProperty = Properties()    
    input = FileInputStream(cmdDefaultSDKPropertyFilePath)
    cmdDefaultSDKProperty.load(input)
    defaultSDKValue = cmdDefaultSDKProperty.getProperty('COMMAND_DEFAULT_SDK')
    input.close()
    defaultSdkProperty = Properties()
    defaultSdkPropertyFilePath = os.path.join(wasInstallRoot,'properties/sdk/' + str(defaultSDKValue) + ".properties")
    input = FileInputStream(defaultSdkPropertyFilePath)
    defaultSdkProperty.load(input)
    defaultSDKPath = defaultSdkProperty.get('com.ibm.websphere.sdk.location.' + str(defaultSDKValue))
    defaultSDKPath = defaultSDKPath.replace('${WAS_INSTALL_ROOT}',wasInstallRoot)
    print "INFO: Default JDK path is " + defaultSDKPath
    input.close()    
    return defaultSDKPath               
     
def doUT(exportDir):    
    exportAADir = os.path.join(exportDir,'exportAA/SCAAlias')    
    exportAuthAliasByName("SCA_Auth_Alias",exportAADir)
    
    exportAADir = os.path.join(exportDir,'exportAA')
    exportAllAuthAliases(exportAADir)
    
    exportDSDir = os.path.join(exportDir,'exportDS/CellScope')
    scopeId = getScopeId("bxv9v447Cell01",None,None,None)
    exportAllDataSources(exportDSDir,scopeId)
    
    exportDSDir = os.path.join(exportDir,'exportDS/NodeScope')
    scopeId = getScopeId("bxv9v447Cell01","bxv9v447Node01",None,None)
    exportAllDataSources(exportDSDir,scopeId)
    
    exportDSDir = os.path.join(exportDir,'exportDS/ClusterScope')
    scopeId = getScopeId("bxv9v447Cell01",None,None,"BPM8011DE.AppTarget")
    exportAllDataSources(exportDSDir,scopeId)
    
    exportSSLDir = os.path.join(exportDir,'exportSSL')
    exportSSLSettings(exportSSLDir)
              
#main()
trace("main()")
trace("Length of param:" + str(len(sys.argv)))
if len(sys.argv) == 1 and sys.argv[0] == "-help":
    usage()

if len(sys.argv) == 2 and sys.argv[0] == "UT":
    doUT(sys.argv[1])
    sys.exit()
     
if len(sys.argv) < 1 and len(sys.argv) > 9:
    usage()
 
if (len(sys.argv) == 1) and sys.argv[0] == "-listDataSources":
    listDataSources()
    sys.exit()
     
if (len(sys.argv) == 1) and sys.argv[0] == "-listAuthAliases":
    listAuthAliases()
    sys.exit()
     
if sys.argv[0] == "-exportAllDataSources" and len(sys.argv) > 2:
    outputDir = sys.argv[1]    
    trace("Length of param:" + str(len(sys.argv)) + "Output Dir:" + outputDir)
    if((len(sys.argv) - 2)%2 != 0):
        usage()
    paramMap = pasrseParam(sys.argv[2:])    
    cellName = paramMap.get("-cell")
    nodeName = paramMap.get("-node")
    serverName = paramMap.get("-server")
    clusterName = paramMap.get("-cluster")
    trace("Params for scope:" + str(paramMap))
    trace("Cell:"+str(cellName)+" Node:"+str(nodeName)+" Server:"+str(serverName)+" Cluster:"+str(clusterName))
    scopeId = getScopeId(cellName,nodeName,serverName,clusterName)
    if(scopeId):
        exportAllDataSources(outputDir,scopeId)
    else:
        usage()  
    sys.exit()      
 
if sys.argv[0] == "-exportDataSource" and len(sys.argv) > 3:
    dsJNDI = sys.argv[1]
    outputDir = sys.argv[2]    
    if((len(sys.argv) - 3)%2 != 0):
        usage()
    paramMap = pasrseParam(sys.argv[3:])
    cellName = paramMap.get("-cell")
    nodeName = paramMap.get("-node")
    serverName = paramMap.get("-server")
    clusterName = paramMap.get("-cluster")    
    trace("Params for scope:"+str(paramMap))
    trace("Cell:"+str(cellName)+" Node:"+str(nodeName)+" Server:"+str(serverName)+" Cluster:"+str(clusterName))
    scopeId = getScopeId(cellName,nodeName,serverName,clusterName)
    if(scopeId):
        exportDataSourceByJNDI(dsJNDI,outputDir,scopeId)
    else:
        usage()
    sys.exit()
     
if sys.argv[0] == "-exportAuthAlias" and len(sys.argv) == 3:
    authAliasName = sys.argv[1]
    outputDir = sys.argv[2]    
    exportAuthAliasByName(authAliasName,outputDir)
    sys.exit()

if sys.argv[0] == "-exportAllAuthAliases" and len(sys.argv) == 2:    
    outputDir = sys.argv[1]    
    exportAllAuthAliases(outputDir)
    sys.exit()
if sys.argv[0] == "-exportSSLSettings" and len(sys.argv) == 2:    
    outputDir = sys.argv[1]    
    exportSSLSettings(outputDir)
    sys.exit()
if sys.argv[0] == "-exportJavaSecurityFile" and len(sys.argv) == 2:    
    outputDir = sys.argv[1]    
    exportJavaSecurityFile(outputDir)
    sys.exit()
    

usage()