#BEGIN COPYRIGHT
#*************************************************************************
#
# Licensed Materials - Property of IBM
# 5725-C94, 5725-C95, 5725-C96
# (C) Copyright IBM Corporation 2014, 2015. All Rights Reserved.
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#*************************************************************************
import os,sys,shutil
import types
import re
from java.util import Properties
from java.io import FileInputStream

debug="false"
#--------------------------------------------------------------
#Print trace according to debug setting
#--------------------------------------------------------------

def trace(message):
    if(debug == "true"):
        print(message)

#------------------------------------------------------------------------------
# Print usage
#------------------------------------------------------------------------------
def usage():
    print "Usage:\n"
    print "\tImport data source configuration contained in import directory by JNDI name:"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -f importWASConfig.py -importDataSource <Datasource_JNDI> <import_dir>\n"
    print "\tImport all data sources configuration contained in import directory:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -f importWASConfig.py -importAllDataSources <import_dir>\n"
    
    print "\tImport J2C authentication configuration contained in import directory by JNDI name:"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -f importWASConfig.py -importAuthAlias <Auth_Alias_Name> <import_dir>\n"
    print "\tImport all J2C authentication configuration contained in import directory by JNDI name:"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -f importWASConfig.py -importAllAuthAliases <import_dir>\n"    
    
    print "\tImport all SSL setting:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -f importWASConfig.py -importAllSSLSettings <import_dir>\n"
    
    print "\tImport SSL setting by alias name:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -f importWASConfig.py -importSSLSetting <SSLSetting_Alias_Name> <import_dir>\n"
    
    print "\tImport java security file java.security:\n"
    print "\t\t<Dmgr_Profile>/bin/wsadmin.sh -conntype NONE -f importWASConfig.py -importJavaSecurityFile <import_dir>\n"
    
    sys.exit()


def isJDBCProviderTypeMatch(objProviderType,jdbcProvider):
    selectedJDBCProviderType = AdminConfig.showAttribute(jdbcProvider,'providerType')
    trace("JDBC provider type of data source that will be imported is:" + str(objProviderType))
    trace("Selected JDBC provider type is:" + str(selectedJDBCProviderType))
    if(selectedJDBCProviderType == objProviderType):
        return "true"
    else:
        print "Selected JDBC provider type does not match the data source that will be imported.JDBC provider type of data source that will be imported is " +str(objProviderType) + " but selected JDBC provider type is " + str(selectedJDBCProviderType) + ", please select JDBC provider again. If there is matched JDBC provider listed, you need create required JDBC provider firstly."
        return "false"
########################################################################
# List all scopes for user select
########################################################################
def listScope():
    scopes = []
    cellList = AdminUtilities.convertToList(AdminConfig.list('Cell'))         
    cellName = AdminConfig.showAttribute(cellList[0],'name') 
    nodeList = AdminUtilities.convertToList(AdminConfig.list('Node'))    
    clusterList = AdminUtilities.convertToList(AdminConfig.list('ServerCluster'))
    index = 1
    print str(index) + ": Cell=" + cellName    
    index = index + 1    
    scopes.append("/Cell:" + cellName + "/")
    for node in nodeList:
        nodeName = AdminConfig.showAttribute(node,'name')
        print str(index) +": Node=" + nodeName
        scopes.append("/Cell:" + cellName + "/Node:"+nodeName + "/")
        index = index + 1
        serverList = AdminUtilities.convertToList(AdminConfig.list('Server',node))
        for server in serverList:
            serverName = AdminConfig.showAttribute(server,'name')
            if((not serverName.startswith("dmgr")) and (not serverName.startswith("nogeagent"))):
                print str(index) + ": Node=" + nodeName + ",Server=" + serverName
                scopes.append("/Cell:" + cellName + "/Node:"+nodeName + "/Server:" + serverName)
                index = index + 1
    for cluster in clusterList:
        clusterName = AdminConfig.showAttribute(cluster,'name')
        print str(index) + ": Cluster=" + clusterName
        scopes.append("/Cell:" + cellName + "/ServerCluster:"+clusterName + "/")
        index = index + 1 
    return scopes

########################################################################
# List all JDBC provider in specified scope
########################################################################
def listJDBCProviderByScope(selectedScope):        
    jdbcProvderList = AdminUtilities.convertToList(AdminConfig.list('JDBCProvider',AdminConfig.getid(selectedScope)))
    if (len(jdbcProvderList) <= 0):
        trace("There is no existed JDBC Provider in selected scope, you need create a new one firstly.")
    else:
        index = 1
        print "Existed JDBC provider in selected scope:"
        for jdbcProvider in jdbcProvderList:
            print str(index) + ": " + AdminConfig.showAttribute(jdbcProvider,'name')
            index = index + 1
    return jdbcProvderList  
        
########################################################################
# Get matched resource adapater in specified scope for creating data source
# If matched not found will use builtine resource adapter
########################################################################
def getResourceAdapterByScope(selectedScope,objRAName):
    matchedResourceAdapter = None
    builtinResourceAdapterID = None
    resourceAdapterList = AdminUtilities.convertToList(AdminConfig.list('J2CResourceAdapter',AdminConfig.getid(selectedScope)))
    if (len(resourceAdapterList) <= 0):
        print "There is no existed resource adapter in selected scope, you need create a new one firstly."
    else:        
        for resourceAdapter in resourceAdapterList:
            if(AdminConfig.showAttribute(resourceAdapter,"name") == objRAName):                
                matchedResourceAdapter = resourceAdapter                
                break
            if(resourceAdapter.find("builtin_rra") > 0):
                trace("Builtin resource adapter was found.")
                builtinResourceAdapterID = resourceAdapter                
    if(matchedResourceAdapter != None):
        print "Resource adapter " + str(objRAName) + " was found, and will be used to create database."
        return matchedResourceAdapter
    else:
        print "Resource adapter " + str(objRAName) + " was not found, and will use built-in resource adapter be used to create database."
        return builtinResourceAdapterID 

def readObjectAttributeFromFile(attributeFilePath):
    attributeFile = open(attributeFilePath, "r")
    objAttributes = attributeFile.read()
    attributeFile.close()
    tmp = objAttributes.splitlines()
    objAttributesString = "".join(tmp)    
    return objAttributesString

def processAttriubutes(objAttributesString):
    regexp = re.compile('''    (                # begin group 1
                            \[                # [
                            \w+\s+            # propertyName followed by spaces/newlines
                            \"[^\"]*        # " followed by a string (non-"-chars)
                            )                # end group 1
                            (\")            # group 2: evil "
                            (                # begin group 3
                            (?!\]+\s*\[).*?    # anything but ]..] [
                            \"\]            # "]
                            )                # end group 3
                        ''', re.DOTALL | re.VERBOSE)
                
                
    lastMatch = re.search(regexp, objAttributesString)
    while lastMatch != "" and lastMatch != None:
        objAttributesString = re.sub(regexp, r"\1'\3", objAttributesString)
        lastMatch = re.search(regexp, objAttributesString)
        
    return objAttributesString

########################################################################
# Replace the string matched the regular expression with new value
########################################################################
def replaceRegexpWithNewValue(originalString,strRegexp,newValue):
    replaceString=originalString
    regexp=re.compile(strRegexp)
    match = re.match(regexp,originalString)
    if(match != "" and match != None):
        originalValue = match.group(1)
        replaceString = originalString.replace(originalValue, newValue)        
    else:
        trace("No match, original string:" + originalString)
    return replaceString
########################################################################
# Create data source with jndi name and exported attributes
########################################################################
def importDataSourceByJNDIAndAttributes(jndiName,attributeFilePath,jdbcProvider,builtinresourceAdapterID):
    
    objAttributesString = readObjectAttributeFromFile(attributeFilePath)
#     objAttributesString = replaceRegexpWithNewValue(objAttributesString,'.+\[provider (\".+JDBCProvider_[0-9]+\)\")\].+',jdbcProvider)
#     objAttributesString = replaceRegexpWithNewValue(objAttributesString,'.+\[relationalResourceAdapter (\".+builtin_rra\)\")\].+',builtinresourceAdapterID)
     
    #Need replace jdbc provider and relationResourceAdapter with actual one in target environment
    trace('Replace "provider" property with actual value in target environment.')
    objAttributesString = replaceRegexpWithNewValue(objAttributesString,'.*\[provider (\[*.*?\]*)\].*',jdbcProvider)
    trace('Replace "relationalResourceAdapter" property with actual value in target environment.')
    objAttributesString = replaceRegexpWithNewValue(objAttributesString,'.*\[relationalResourceAdapter (\[*.*?\]*)\].*',builtinresourceAdapterID)                   
    objAttributesString = processAttriubutes(objAttributesString)
                
    print "Createing dataSource..."
    result = AdminConfig.create("DataSource", jdbcProvider, "[" + objAttributesString + "]")
    AdminConfig.save()    

def updateExistedDataSource(existedDataSource,attributeFilePath):
    objAttributesString = readObjectAttributeFromFile(attributeFilePath)
    
    #As following three properties is read-only that can not be updated, so need remove them from exported attributes
    trace('Remove property "provider" from imported data source configuration as it is read-only property that can not be update.')
    objAttributesString = replaceRegexpWithNewValue(objAttributesString,'.*(\[provider .*?\]).*',"")
    trace('Remove property "relationalResourceAdapter" from imported data source configuration as it is read-only property that can not be update.')
    objAttributesString = replaceRegexpWithNewValue(objAttributesString,'.*(\[relationalResourceAdapter .*?\]).*',"")      
    trace('Remove property "providerType" from imported data source configuration as it is read-only property that can not be update.')
    objAttributesString = replaceRegexpWithNewValue(objAttributesString,'.*(\[providerType .*?\]).*',"")
    
    objAttributesString = processAttriubutes(objAttributesString)
    AdminConfig.modify(existedDataSource,"[" + objAttributesString + "]")
    AdminConfig.save()
    
########################################################################
# Create data source by JNDI name
########################################################################                          
def importDataSourceByJNDI(jndiName, attributeFilePath,objProviderName,objProviderType,objRAName):
    existedDataSource = dataSourceExisted(jndiName)
    if(existedDataSource != None):
        overwrite = raw_input("Data source with JNDI name " + jndiName + " already existed. Would you like to overwrite its configuration?(Y/N)")
        if(overwrite == "Y" or overwrite == "y"):
            updateExistedDataSource(existedDataSource,attributeFilePath)
            print "Data source with JNDI name " + jndiName + " was updated successfully."
        else:
            print "Import of data source with JNDI name was skipped."            
    else:    
        print "Data source with JNDI name " + jndiName + " did not existed. Will create it."    
        selectedScope = getUserSelectedScope()
        if(selectedScope != None):
            builtinresourceAdapterID= getResourceAdapterByScope(selectedScope,objRAName)    
            trace("Found builtin resource adapter in selected scope:" + str(builtinresourceAdapterID))
            jdbcProvider = getUserSelectedJDBCProvider(selectedScope)
            while isJDBCProviderTypeMatch(objProviderType,jdbcProvider) != "true":
                jdbcProvider = getUserSelectedJDBCProvider(selectedScope)                         
            if(jdbcProvider != None and builtinresourceAdapterID != None ):
                importDataSourceByJNDIAndAttributes(jndiName,attributeFilePath,jdbcProvider,builtinresourceAdapterID)
                print "Data source with JNDI name " + jndiName + " was imported successfully."            

########################################################################
# Prompt user to select scope where to create data source
########################################################################            
def getUserSelectedScope():
    scopes = listScope()    
    selectScopeIndex = raw_input("Please input the number to select the scope where you want to create data source:")
    isCorrectNumber = 'false'
    while isCorrectNumber == 'false':
        try:
            selectScopeIndex = int(selectScopeIndex)
            if(selectScopeIndex >= 1 and selectScopeIndex <= len(scopes)):
                isCorrectNumber = 'true'                          
            else:
                selectScopeIndex = raw_input("Incorrect number! Please input the number select the scope where you want to create data source:")                     
        except ValueError:
            selectScopeIndex = raw_input("Incorrect number! Please input the number select the scope where you want to create data source:")    
    trace("selected scope index " + str(selectScopeIndex) + ": " + scopes[selectScopeIndex - 1])
    
    return scopes[selectScopeIndex - 1]   

########################################################################
# Prompt user to select JDBC provider for creating data source
######################################################################## 
def getUserSelectedJDBCProvider(selectedScope):    
    jdbcProvider = None
    jdbcProvderList = listJDBCProviderByScope(selectedScope)
    if(len(jdbcProvderList) > 0):
        selectJDBCProviderIndex = raw_input("Please input the number to select the JDBC Provider for data source:")
        isCorrectNumber = 'false'
        while isCorrectNumber == 'false':
            try:
                selectJDBCProviderIndex = int(selectJDBCProviderIndex)
                if(selectJDBCProviderIndex >= 1 and selectJDBCProviderIndex <= len(jdbcProvderList)):
                    isCorrectNumber = 'true'         
                else:
                    selectJDBCProviderIndex = raw_input("Incorrect number!Please input the number to select the JDBC Provider for data source:")                           
            except ValueError:
                selectJDBCProviderIndex = raw_input("Incorrect number!Please input the number to select the JDBC Provider for data source:")
        
        trace("selected JDBC Provider " + str(selectJDBCProviderIndex) + ": " + jdbcProvderList[selectJDBCProviderIndex - 1])
        jdbcProvider = jdbcProvderList[selectJDBCProviderIndex - 1]
    else:
        print( "There is no existed JDBC Provider in selected scope, you need create required JDBC Provider for data source firstly.")
        sys.exit()
    return jdbcProvider

def checkImportDirExisted(importDir):
    if(not os.path.exists(importDir)):
        print "Import directory " + importDir + " does not exist, please check and try again."
        sys.exit()
        
def importAuthAliasByName(authAliasName, attributeFilePath):
    existedAlias = None
    cellList = AdminUtilities.convertToList(AdminConfig.list('Cell'))         
    cellName = AdminConfig.showAttribute(cellList[0],'name') 
    
    security = AdminConfig.list("Security")
    objAttributesString = readObjectAttributeFromFile(attributeFilePath)    
    objAttributesString = processAttriubutes(objAttributesString)    
    authAliasList = AdminUtilities.convertToList(AdminConfig.list("JAASAuthData",security))
    for authAlias in authAliasList:
        if(AdminConfig.showAttribute(authAlias,"alias") == authAliasName):
            existedAlias = authAlias
            break
    if(existedAlias == None ):
        print "Creating J2C authentication..."   
        AdminConfig.create('JAASAuthData', security, "[" + objAttributesString + "]")
        AdminConfig.save()
        print "Import of J2C authentication data configuration was completed successfully."
    else:
        overwrite = raw_input("J2C authentication alias with name " + authAliasName + " already existed. Would you like to overwrite it?(Y/N)")
        if(overwrite == "Y" or overwrite == "y"):
            AdminConfig.modify(existedAlias, "[" + objAttributesString + "]")
            AdminConfig.save()
            print "J2C authentication with name " + authAliasName + " was updated successfully."  
        else:
            print "Import of J2C authentication with name " + authAliasName + " will be skipped."    
           
        
def printPrompt(objType):
    if(objType == 0):
        answer = raw_input("If the data source configuration you're about to import refers to some non-default resources like J2C authentication data or resource adapter,JDBC provider, please make sure you have already created those referred resource firstly.Continue?(Y/N):")
        if (answer != 'Y' and answer != 'y' ):
            sys.exit()    

def importSpecifiedDataSource(importDir,jndiName):
    checkImportDirExisted(importDir)
    printPrompt(0)    
    dirList = os.listdir(importDir)
    for fileName in dirList:
        if (fileName == "exportDS.properties"):
            propsFilePath = os.path.join(importDir,fileName)
            propsFile = Properties()    
            inputSream = FileInputStream(propsFilePath)
            propsFile.load(inputSream)
            objType = propsFile.getProperty(jndiName + ".ObjectType")
            objFileName = propsFile.getProperty(jndiName + ".ObjectFileName")
            objProviderName = propsFile.getProperty(jndiName + ".Provider")
            objProviderType = propsFile.getProperty(jndiName + ".ProviderType")
            objRAName = propsFile.getProperty(jndiName + ".ResourceAdapter")
            inputSream.close()           
            if (objType):
                if(objType == "DataSource"):
                    print "Importing data source with JNDI name " + jndiName
                    importDataSourceByJNDI(jndiName,os.path.join(importDir,objFileName),objProviderName,objProviderType,objRAName)                         
                else:
                    print "The object with JNDI name " + jndiName +" you want to import is not data source type, please check and try again."
            else:
                print "The object with JNDI name " + jndiName +" you want to import could not be found in import directory " + importDir + "." 

def importAllDataSources(importDir):
    checkImportDirExisted(importDir)
    printPrompt(0) 
    trace("importDir: " + importDir)
    dirList = os.listdir(importDir)
    for fileName in dirList:
        if (fileName == "exportDS.properties"):
            jndiAttributeMap = {}
            propsFilePath = os.path.join(importDir,fileName)
            propsFile = Properties()    
            inputSream = FileInputStream(propsFilePath)
            propsFile.load(inputSream)
            for key in propsFile.keySet():
                trace("Key: " + str(key))
                index = str(key).find(".ObjectFileName")
                if(index > 0):
                    jndiName = str(key)[0:index]
                    trace("jndiName: " + jndiName)
                    attributeFileName = propsFile.getProperty(str(key))
                    trace("attributeFileName: " + attributeFileName)
                    if(propsFile.getProperty(jndiName + ".ObjectType") == "DataSource"):
                        objProviderName = propsFile.getProperty(jndiName + ".Provider")
                        objProviderType = propsFile.getProperty(jndiName + ".ProviderType")
                        objRAName = propsFile.getProperty(jndiName + ".ResourceAdapter")
                        print "Importing data source with JNDI name " + jndiName
                        importDataSourceByJNDI(jndiName,os.path.join(importDir,attributeFileName),objProviderName,objProviderType,objRAName)            
                        print "All import of data source configuration was completed successfully."            
                    else:
                        print "The object with JNDI name " + jndiName +" you want to import is not data source type or could not be found in import directory, please check and try again."
    
def importSpecifiedAuthAliasByName(importDir,authAliasName):
    checkImportDirExisted(importDir)
    printPrompt(1)
    dirList = os.listdir(importDir)
    for fileName in dirList:
        if (fileName == "exportAlias.properties"):
            propsFilePath = os.path.join(importDir,fileName)
            propsFile = Properties()    
            inputSream = FileInputStream(propsFilePath)
            propsFile.load(inputSream)
            objType = propsFile.getProperty(authAliasName + ".ObjectType")
            objFileName = propsFile.getProperty(authAliasName + ".ObjectFileName")
            inputSream.close()           
            if (objType):
                if(objType == "JAASAuthData"):
                    print "Importing J2C authentication with name " + authAliasName
                    importAuthAliasByName(authAliasName,os.path.join(importDir,objFileName))                   
                else:
                    print "The object with name " + authAliasName +" you want to import is not J2C authentication type, please check and try again."                               
            else:
                print "The object with name " + authAliasName +" you want to import could not be found in import directory " + importDir 
    

def importAllAuthAliases(importDir):
    checkImportDirExisted(importDir)
    printPrompt(1)
    trace("importDir: " + importDir)
    dirList = os.listdir(importDir)
    for fileName in dirList:
        if (fileName == "exportAlias.properties"):
            jndiAttributeMap = {}
            propsFilePath = os.path.join(importDir,fileName)
            propsFile = Properties()    
            inputSream = FileInputStream(propsFilePath)
            propsFile.load(inputSream)
            for key in propsFile.keySet():
                trace("Key: " + str(key))
                index = str(key).find(".ObjectFileName")
                if(index > 0):
                    authAiasName = str(key)[0:index]
                    trace("authAiasName: " + authAiasName)
                    attributeFileName = propsFile.getProperty(str(key))
                    trace("attributeFileName: " + attributeFileName)
                    if(propsFile.getProperty(authAiasName + ".ObjectType") == "JAASAuthData"):
                        print "Importing J2C authentication with name " + authAiasName
                        importAuthAliasByName(authAiasName,os.path.join(importDir,attributeFileName))
                        print "All import of J2C authentication data configuration was completed successfully."
                    else:
                        print "The object with name " + authAiasName +" you want to import is not J2C authentication type or could not be found in import directory, please check and try again."

def importSpecifiedSSLSetting(importDir,sslConfigAliasName):    
    checkImportDirExisted(importDir)
    sslSettingFile = None
    dirList = os.listdir(importDir)
    for fileName in dirList:
        if (fileName == "exportSSL.properties"):
            jndiAttributeMap = {}
            propsFilePath = os.path.join(importDir,fileName)
            propsFile = Properties()    
            inputSream = FileInputStream(propsFilePath)
            propsFile.load(inputSream)
            for key in propsFile.keySet():
                trace("Key: " + str(key))
                if(str(key) == sslConfigAliasName + ".ObjectFileName"):
                    sslSettingFile = propsFile.getProperty(str(key))
                    break;                 
    if(sslSettingFile != None):
        sslConfigs = AdminUtilities.convertToList(AdminTask.listSSLConfigs('[-all true -displayObjectName true ]'))    
        if(len(sslConfigs) > 0):
            for sslConfig in sslConfigs:
                alias = AdminConfig.showAttribute(sslConfig,"alias")
                if(alias == sslConfigAliasName):
                    print "Importing setting for SSL configuration - " + alias
                    updateExistedSSLSetting(AdminConfig.showAttribute(sslConfig,"setting"),os.path.join(importDir,sslSettingFile))
                    print "Import setting for SSL configuration - " + alias + " was completed."      
                    break
    else:
        print "Setting for SSL configuration with alias name " + sslConfigAliasName + " was not found in import directory " + importDir + ", please check import directory and try again."
        
    
def importSSLSetting(importDir):   
    checkImportDirExisted(importDir) 
    sslSettingMap = {}
    dirList = os.listdir(importDir)
    for fileName in dirList:
        if (fileName == "exportSSL.properties"):
            jndiAttributeMap = {}
            propsFilePath = os.path.join(importDir,fileName)
            propsFile = Properties()    
            inputSream = FileInputStream(propsFilePath)
            propsFile.load(inputSream)
            for key in propsFile.keySet():
                trace("Key: " + str(key))
                index = str(key).find(".ObjectFileName")
                if(index > 0):
                    sslSettingMap[str(key)] = propsFile.getProperty(str(key))
                       
    sslConfigs = AdminUtilities.convertToList(AdminTask.listSSLConfigs('[-all true -displayObjectName true ]'))    
    if(len(sslConfigs) > 0):
        for sslConfig in sslConfigs:
            alias = AdminConfig.showAttribute(sslConfig,"alias")
            if(sslSettingMap.has_key(alias + ".ObjectFileName")):
                print "Importing setting for SSL configuration - " + alias
                updateExistedSSLSetting(AdminConfig.showAttribute(sslConfig,"setting"),os.path.join(importDir,sslSettingMap[alias + ".ObjectFileName"]))
                print "Import setting for SSL configuration - " + alias + " was completed."  

def updateExistedSSLSetting(sslConfigSetting, attributeFilePath):
    #Read attributes for ssl setting that will be imported from file
    sslAttributeString = readObjectAttributeFromFile(attributeFilePath)
    #Replace 'keyManager' in SSL Setting by existed one in target environment as exported one from source environment is not available in target
    existedKeyManager = AdminConfig.showAttribute(sslConfigSetting,"keyManager")
    if(existedKeyManager != None and existedKeyManager != ""):
        sslAttributeString = replaceRegexpWithNewValue(sslAttributeString,'.*\[keyManager (\[*.*?\]*)\].*',existedKeyManager)
    
    #Replace 'keyStore' in SSL Setting by existed one in target environment as exported one from source environment is not available in target
    existedKeyStore = AdminConfig.showAttribute(sslConfigSetting,"keyStore")
    if(existedKeyStore != None and existedKeyStore != ""):
        sslAttributeString = replaceRegexpWithNewValue(sslAttributeString,'.*\[keyStore (\[*.*?\]*)\].*',existedKeyStore)
    
    #Replace 'trustManager' in SSL Setting by existed one in target environment as exported one from source environment is not available in target
    existedtrustManager = AdminConfig.showAttribute(sslConfigSetting,"trustManager")
    if(existedtrustManager != None and existedtrustManager != ""):
        sslAttributeString = replaceRegexpWithNewValue(sslAttributeString,'.*\[trustManager (\[*.*?\]*)\].*',existedtrustManager)
    
    #Replace 'trustStore' in SSL Setting by existed one in target environment as exported one from source environment is not available in target
    existedtruststore = AdminConfig.showAttribute(sslConfigSetting,"trustStore")
    if(existedtruststore != None and existedtruststore != ""):
        sslAttributeString = replaceRegexpWithNewValue(sslAttributeString,'.*\[trustStore (\[*.*?\]*)\].*',existedtruststore)    
    
    sslAttributeString = processAttriubutes(sslAttributeString)
        
    AdminConfig.modify(sslConfigSetting,"[" + sslAttributeString + "]")
    AdminConfig.save()

          
########################################################################
# Check if data source with specified JNDI name existed
########################################################################
def dataSourceExisted(jndiName):
    dataSourceList=AdminConfig.list("DataSource")
    dataSources = AdminUtilities.convertToList(dataSourceList)    
    matchedDS = None
    for datasource in dataSources:
        tmpJNDIName=AdminConfig.showAttribute(datasource,"jndiName")
        if (tmpJNDIName != None and tmpJNDIName == jndiName):     
            matchedDS = datasource
            break;
    return matchedDS

def importJavaSecurityFile(importDir):    
    checkImportDirExisted(importDir)    
    defaultJDKPath = getDefaultJDKPath()
    targetJavaSecurityFilePath = os.path.join(defaultJDKPath,'jre/lib/security/java.security')
    sourceJavaSecurityFilePath = os.path.join(importDir,'java.security')
    mergeJavaSecurityFileContent(sourceJavaSecurityFilePath,targetJavaSecurityFilePath)
    #shutil.copyfile(importFilePath, jreSecurityFilePath)
    print "INFO:" + sourceJavaSecurityFilePath + " was merged into " + targetJavaSecurityFilePath

def mergeJavaSecurityFileContent(sourceJavaSecurityFilePath,targetJavaSecurityFilePath):
    from java.util import Properties
    from java.io import FileInputStream
    #Backup java.security in target env
    targetJavaSecurityFilePathBak = str(targetJavaSecurityFilePath) + ".bak"
    if(not os.path.exists(targetJavaSecurityFilePathBak)):
        print "INFO: Backup " + targetJavaSecurityFilePath + " to " + targetJavaSecurityFilePathBak
        shutil.copy(targetJavaSecurityFilePath,targetJavaSecurityFilePathBak)
    else:
        print "INFO: java.security file was already backup to " + targetJavaSecurityFilePathBak
    #Load source java.security content into a Properties object
    sourceJavaSecurityProperties = Properties()
    input = FileInputStream(sourceJavaSecurityFilePath)
    sourceJavaSecurityProperties.load(input)
    input.close()
    #Load target java.security content into a Properties object
    targetJavaSecurityProperties = Properties()
    input = FileInputStream(targetJavaSecurityFilePath)
    targetJavaSecurityProperties.load(input)
    input.close()
    #Process target java.security line by line and write to another temp file
    targetJavaSecurityFile = open(targetJavaSecurityFilePath,"r")
    tmpTargetJavaSecurityFile = open(str(targetJavaSecurityFilePath)+".tmp","w")
    while 1:
        line = targetJavaSecurityFile.readline()
        if (not line):
            break        
        #If the line is comment line write it into temp file without any change
        if (line.startswith("# ")):
            tmpTargetJavaSecurityFile.write(line)
        #If the line is commented property line, check if the property already existed in target or source java.security
        #If existed in target we will handle it in next step, if not will check if existed in source, if the commented property file
        # existed in source , will write the property with value set in source into temp file
        elif (line.startswith("#") and line.find("=") != -1):
            key = line[1:line.find("=")]            
            if(not targetJavaSecurityProperties.containsKey(key) and sourceJavaSecurityProperties.containsKey(key)):
                tmpTargetJavaSecurityFile.write(key + "=" + str(sourceJavaSecurityProperties.get(key)) + "\n")
                sourceJavaSecurityProperties.put(key,"None")
            else:
                tmpTargetJavaSecurityFile.write(line)
        #Process property line, if the property existed in source will replace the property value with source value and write to temp file
        elif (line.find("=") != -1):
            key = line[0:line.find("=")]            
            if(sourceJavaSecurityProperties.containsKey(key)):
                tmpTargetJavaSecurityFile.write(key + "=" + str(sourceJavaSecurityProperties.get(key))+ "\n")
                sourceJavaSecurityProperties.put(key,"None")
            else:
                tmpTargetJavaSecurityFile.write(line)
        else:
            tmpTargetJavaSecurityFile.write(line)      
    # Append all properties existed in source java.security file but not existed in target  
    for key in sourceJavaSecurityProperties.keySet():
        if(sourceJavaSecurityProperties.get(key) != "None"):
            tmpTargetJavaSecurityFile.write(key + "=" + str(sourceJavaSecurityProperties.get(key))+ "\n")             
    targetJavaSecurityFile.close()
    tmpTargetJavaSecurityFile.close()
    #Replace target java.security with temp file and delete the temp file
    shutil.copy(str(targetJavaSecurityFilePath)+".tmp",targetJavaSecurityFilePath)
    os.remove(str(targetJavaSecurityFilePath)+".tmp")   
                      
def getDefaultJDKPath():      
    from java.util import Properties
    from java.io import FileInputStream
    wasInstallRoot = java.lang.System.getProperty('was.install.root')
    cmdDefaultSDKPropertyFilePath = os.path.join(wasInstallRoot,'properties/sdk/cmdDefaultSDK.properties')
    cmdDefaultSDKProperty = Properties()    
    input = FileInputStream(cmdDefaultSDKPropertyFilePath)
    cmdDefaultSDKProperty.load(input)
    defaultSDKValue = cmdDefaultSDKProperty.getProperty('COMMAND_DEFAULT_SDK')
    input.close()
    defaultSdkProperty = Properties()
    defaultSdkPropertyFilePath = os.path.join(wasInstallRoot,'properties/sdk/' + str(defaultSDKValue) + ".properties")
    input = FileInputStream(defaultSdkPropertyFilePath)
    defaultSdkProperty.load(input)
    defaultSDKPath = defaultSdkProperty.get('com.ibm.websphere.sdk.location.' + str(defaultSDKValue))
    defaultSDKPath = defaultSDKPath.replace('${WAS_INSTALL_ROOT}',wasInstallRoot)
    print "INFO: Default JDK path is " + defaultSDKPath
    input.close()    
    return defaultSDKPath
                
############################################################################
# importDir should be a folder that contains three sub-folder with name 
# 'exportDS','exportAA','exportSSL'
############################################################################         
def doUT(importDir):
    toImportDSJNDI = 'jdbc/WPSDB'
    toImportAAName = 'WPSDB_Auth_Alias'
    toImportSSLName = 'CellDefaultSSLSettings'
    exportDSDir = os.path.join(importDir,'exportDS')
    exportAADir = os.path.join(importDir,'exportAA')
    exportSSLDir = os.path.join(importDir,'exportSSL')
    print "Testing importDataSourceByJNDIName"
    importSpecifiedDataSource(exportDSDir,toImportDSJNDI)
    print "Testing importDataSourceByJNDIName was done"
    
    print "Testing importAllDataSources was done"
    importAllDataSources(exportDSDir)
    print "Testing importAllDataSources was done"
    
    print "Testing importAllAuthAliases was done"
    importAllAuthAliases(exportAADir)
    print "Testing importAllAuthAliases was done"
    
    print "Testing importSpecifiedAuthAliasByName was done"
    importSpecifiedAuthAliasByName(exportAADir,toImportAAName)
    print "Testing importSpecifiedAuthAliasByName was done"    
    
    print "Testing importSpecifiedAuthAliasByName was done"
    toImportAAName = 'PROCSVR_BPM8011DE.AppTarget_Auth_Alias'
    importSpecifiedAuthAliasByName(exportAADir,toImportAAName)
    print "Testing importSpecifiedAuthAliasByName was done"
    
    print "Testing importSSLSettings was done"
    importSSLSetting(exportSSLDir)
    print "Testing importSSLSettings was done"
    
    print "Testing importSpecifiedSSLSetting was done"
    importSpecifiedSSLSetting(exportSSLDir,toImportSSLName)
    print "Testing importSpecifiedSSLSetting was done"
    print "All UT case was passed."
########################################################################
# main()
######################################################################## 
trace("main()")

if len(sys.argv) == 1 and sys.argv[0] == "help":
    usage()

if len(sys.argv) == 2 and sys.argv[0] == "UT":
    doUT(sys.argv[1])
    sys.exit()
         
if len(sys.argv) < 1 and len(sys.argv) > 3:
    usage()
 
if (len(sys.argv) == 3) and sys.argv[0] == "-importDataSource":
    importDir = sys.argv[2]
    jndiName = sys.argv[1]
    importSpecifiedDataSource(importDir,jndiName)
    sys.exit()

if (len(sys.argv) == 2) and sys.argv[0] == "-importAllDataSources":
    importDir = sys.argv[1]
    importAllDataSources(importDir)
    sys.exit()

     
if (len(sys.argv) == 3) and sys.argv[0] == "-importAuthAlias":
    authAliasName = sys.argv[1]
    importDir = sys.argv[2]
    importSpecifiedAuthAliasByName(importDir,authAliasName)
    sys.exit()
    

if (len(sys.argv) == 2) and sys.argv[0] == "-importAllAuthAliases":
    importDir = sys.argv[1]
    importAllAuthAliases(importDir)
    sys.exit()    

if (len(sys.argv) == 2) and sys.argv[0] == "-importAllSSLSettings":
    importDir = sys.argv[1]
    print "Importing SSL configuration setting."
    importSSLSetting(importDir)   
    print "All import of SSL configuration setting was completed successfully."
    sys.exit()

if (len(sys.argv) == 3) and sys.argv[0] == "-importSSLSetting":
    importDir = sys.argv[2]
    sslConfigAliasName = sys.argv[1]
    print "Importing SSL configuration setting."
    importSpecifiedSSLSetting(importDir,sslConfigAliasName)   
    print "Import of SSL configuration setting was completed successfully."
    sys.exit()
    
if (len(sys.argv) == 2) and sys.argv[0] == "-importJavaSecurityFile":
    importDir = sys.argv[1]
    print "Importing Java Security file java.security."
    importJavaSecurityFile(importDir)    
    sys.exit()

usage()
